@@ -46,8 +46,8 @@
 typedef boost::shared_ptr<KCal::Incidence> IncidencePtr;
 
 extern "C" {
-#include <gcalendar.h>
-#include <gcal_status.h>
+#include <libgcal/gcalendar.h>
+#include <libgcal/gcal_status.h>
 }
 
 using namespace Akonadi;
@@ -89,6 +89,16 @@ void GCalResource::saveTimestamp(QString &timestamp)
 	Settings::self()->writeConfig();
 }
 
+void GCalResource::retrieveUsername(QString &username)
+{
+	username = Settings::self()->username();
+}
+
+void GCalResource::saveUsername(QString &username)
+{
+	Settings::self()->setUsername(username);
+	Settings::self()->writeConfig();
+}
 
 void GCalResource::retrieveCollections()
 {
@@ -141,7 +151,7 @@ void GCalResource::retrieveItems( const Akonadi::Collection &collection )
 	kError() << "First retrieve";
 
 	if ((result = gcal_get_events(gcal, &all_events))) {
-		ResourceBase::cancelTask(i18n("Failed events retrieving!"));
+		ResourceBase::cancelTask(i18n("Failed to retrieve events."));
 		ResourceBase::doSetOnline(false);
 		return;
 	}
@@ -221,6 +231,7 @@ void GCalResource::aboutToQuit()
 {
 	// TODO: any cleanup you need to do while there is still an active
 	// event loop. The resource will terminate after this method returns
+    gcal_final_cleanup();
 }
 
 void GCalResource::doSetOnline(bool online)
@@ -231,14 +242,19 @@ void GCalResource::doSetOnline(bool online)
 	QString password;
 	int result = 0;
 	WId window = winIdForDialogs();
+        retrieveUsername(user);
 
-	if (online)
-		if (!retrieveFromWallet(user, password, window))
-			if (!(result = authenticate(user, password))) {
-				authenticated = true;
-				ResourceBase::doSetOnline(online);
-				synchronize();
-			}
+	if (online) {
+            if (!user.isEmpty()) {
+                if (!retrieveFromWallet(user, password, window,
+                                        QString("gcal")))
+                    if (!(result = authenticate(user, password))) {
+                        authenticated = true;
+                        ResourceBase::doSetOnline(online);
+                        synchronize();
+                    }
+            }
+        }
 
 	if (result) {
 		kError() << "Failed setting online.";
@@ -425,8 +441,11 @@ void GCalResource::configure( WId windowId )
 	}
 
 	const int walletRes = saveToWallet(dlgConf->eAccount->text(),
-				     dlgConf->ePass->text(),
-				     windowId);
+                                           dlgConf->ePass->text(),
+                                           windowId, QString("gcal"));
+        QString user(dlgConf->eAccount->text());
+        saveUsername(user);
+
 	if (walletRes)
 		kError() << "Cannot save user info: is user using kwallet?.";
 
@@ -452,7 +471,7 @@ void GCalResource::itemAdded( const Akonadi::Item &item, const Akonadi::Collecti
 					      " use of Google calendar.");
 		emit error(message);
 		emit status(Broken, message);
-		ResourceBase::cancelTask(i18n("Failed adding event!"));
+		ResourceBase::cancelTask(i18n("Failed to add event."));
 		ResourceBase::doSetOnline(false);
 		return;
 	}
@@ -518,7 +537,7 @@ void GCalResource::itemAdded( const Akonadi::Item &item, const Akonadi::Collecti
 					      "Failed adding new calendar.");
 		emit error(message);
 		emit status(Broken, message);
-		ResourceBase::cancelTask(i18n("Failed adding event!"));
+		ResourceBase::cancelTask(i18n("Failed to add event."));
 		ResourceBase::doSetOnline(false);
 	}
 
@@ -549,7 +568,7 @@ void GCalResource::itemChanged( const Akonadi::Item &item, const QSet<QByteArray
 					      " use of Google calendar.");
 		emit error(message);
 		emit status(Broken, message);
-		ResourceBase::cancelTask(i18n("Failed editing event!"));
+		ResourceBase::cancelTask(i18n("Failed to edit event."));
 		ResourceBase::doSetOnline(false);
 
 		return;
@@ -623,7 +642,7 @@ void GCalResource::itemChanged( const Akonadi::Item &item, const QSet<QByteArray
 					      "Failed adding new calendar.");
 		emit error(message);
 		emit status(Broken, message);
-		ResourceBase::cancelTask(i18n("Failed editing event!"));
+		ResourceBase::cancelTask(i18n("Failed to edit event."));
 		ResourceBase::doSetOnline(false);
 	}
 
@@ -655,7 +674,7 @@ void GCalResource::itemRemoved( const Akonadi::Item &item )
 					      "use of Google calendar.");
 		emit error(message);
 		emit status(Broken, message);
-		ResourceBase::cancelTask(i18n("Failed deleting event!"));
+		ResourceBase::cancelTask(i18n("Failed to delete event."));
 		ResourceBase::doSetOnline(false);
 	}
 
