@@ -2,6 +2,7 @@
 /* gcalresource.h 						       */
 /* 								       */
 /* Copyright (C) 2009  Adenilson Cavalcanti <savagobr@yahoo.com>       */
+/* Copyright (C) 2010  Holger Kral <holger.kral@gmx.net>               */
 /* 								       */
 /* This library is free software; you can redistribute it and/or       */
 /* modify it under the terms of the GNU Lesser General Public	       */
@@ -46,14 +47,21 @@
 #include <KUrl>
 
 extern "C" {
-#include <gcalendar.h>
-#include <gcontact.h>
-#include <gcal_status.h>
+#include <libgcal/gcalendar.h>
+#include <libgcal/gcontact.h>
+#include <libgcal/gcal_status.h>
 }
 
 
 using namespace Akonadi;
 
+QString GoogleContactsResource::extractStructuredField(gcal_structured_subvalues_t structured_entry, char *fieldName, int index1, int index2)
+{
+    QString res;
+    res = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry, index1, index2, fieldName));
+    return res;
+}
+
 GoogleContactsResource::GoogleContactsResource( const QString &id )
 	: ResourceBase(id)
 {
@@ -93,6 +101,16 @@ void GoogleContactsResource::saveTimestamp(QString &timestamp)
  	Settings::self()->writeConfig();
 }
 
+void GoogleContactsResource::retrieveUsername(QString &username)
+{
+	username = Settings::self()->username();
+}
+
+void GoogleContactsResource::saveUsername(QString &username)
+{
+	Settings::self()->setUsername(username);
+	Settings::self()->writeConfig();
+}
 
 void GoogleContactsResource::retrieveCollections()
 {
@@ -158,7 +176,7 @@ KABC::PhoneNumber::Type googlePhoneLabelToAkonadiType(gcal_phone_type label) {
 }
 
 KABC::Address::Type googleAddressLabelToAkonadiType(gcal_address_type label) {
-	switch ( label ) {
+	switch (label) {
 		case A_HOME:
 			return KABC::Address::Home;
 			break;
@@ -226,6 +244,8 @@ void GoogleContactsResource::retrieveItems( const Akonadi::Collection &collectio
 	saveTimestamp(timestamp);
 
 
+        QStringList listImTypes;
+        listImTypes << "AIM" << "MSN" << "YAHOO" << "SKYPE" << "ICQ" << "JABBER" << "QQ";
 	/* Each google entry has a unique ID and edit_url */
 	for (size_t i = 0; i < all_contacts.length; ++i) {
 		Item item(QLatin1String("text/directory"));
@@ -233,6 +253,7 @@ void GoogleContactsResource::retrieveItems( const Akonadi::Collection &collectio
 
 		KABC::Addressee addressee;
 		KABC::Picture photo;
+		KABC::Address::Type temp_address_type;
 		QImage image;
 		QString temp;
 		QDateTime tempDate;
@@ -241,46 +262,52 @@ void GoogleContactsResource::retrieveItems( const Akonadi::Collection &collectio
 		int structured_entry_count;
 		int j;
 		bool fill_entry;
-		
+
 		/* structured name */
 		structured_entry = gcal_contact_get_structured_name(contact);
 		fill_entry = 1;
-		temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,0,1,"givenName"));
+		temp = extractStructuredField(structured_entry, "givenName");
 		addressee.setGivenName(temp);
-		if(temp.length())
+		if (temp.length())
 			fill_entry = 0;
-		temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,0,1,"additionalName"));
+
+		temp = extractStructuredField(structured_entry, "additionalName");
 		addressee.setAdditionalName(temp);
-		if(temp.length())
+		if (temp.length())
 			fill_entry = 0;
-		temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,0,1,"familyName"));
+
+		temp = extractStructuredField(structured_entry, "familyName");
 		addressee.setFamilyName(temp);
-		if(temp.length())
+		if (temp.length())
 			fill_entry = 0;
-		temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,0,1,"namePrefix"));
+
+		temp = extractStructuredField(structured_entry, "namePrefix");
 		addressee.setPrefix(temp);
-		if(temp.length())
+		if (temp.length())
 			fill_entry = 0;
-		temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,0,1,"nameSuffix"));
+
+		temp = extractStructuredField(structured_entry, "nameSuffix");
 		addressee.setSuffix(temp);
-		if(temp.length())
+		if (temp.length())
 			fill_entry = 0;
-		if(fill_entry)
-		{
-			temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,0,1,"fullName"));
+
+		if (fill_entry) {
+			temp = extractStructuredField(structured_entry, "fullName");
 			addressee.setNameFromString(temp);
-			if(temp.length())
+			if (temp.length())
 				fill_entry = 0;
 		}
-		if(fill_entry)
-		{
+
+		if (fill_entry) {
 			/* name */
 			temp = QString::fromUtf8(gcal_contact_get_title(contact));
 			addressee.setNameFromString(temp);
 		}
+
 		/* nickname */
 		temp = QString::fromUtf8(gcal_contact_get_nickname(contact));
 		addressee.setNickName(temp);
+
 		/* email */
 		for (j = 0; j < gcal_contact_get_emails_count(contact); j++) {
 			temp = QString::fromUtf8(gcal_contact_get_email_address(contact, j));
@@ -289,46 +316,55 @@ void GoogleContactsResource::retrieveItems( const Akonadi::Collection &collectio
 			addressee.insertCustom(QString::fromUtf8("Google"),
 				QString::fromUtf8("typeof_email_").append(QString::number(j)), temp);
 		}
-		
+
 		/* address */
 		structured_entry = gcal_contact_get_structured_address(contact);
 		structured_entry_count = gcal_contact_get_structured_address_count(contact);
 		for (j = 0; j < structured_entry_count; j++) {
 			KABC::Address address;
 			fill_entry = 1;
-			address.setType(googleAddressLabelToAkonadiType(gcal_contact_get_structured_address_type(contact,j,structured_entry_count)));
-			temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,j,structured_entry_count,"street"));
+			temp_address_type = googleAddressLabelToAkonadiType(gcal_contact_get_structured_address_type(contact, j, structured_entry_count));
+			if (j == gcal_contact_get_pref_structured_address(contact))
+				temp_address_type = temp_address_type | KABC::Address::Pref;
+			address.setType(temp_address_type);
+
+                        temp = extractStructuredField(structured_entry, "street", j, structured_entry_count);
 			address.setStreet(temp);
-			if(temp.length())
+			if (temp.length())
 				fill_entry = 0;
-			temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,j,structured_entry_count,"pobox"));
+
+                        temp = extractStructuredField(structured_entry, "pobox", j, structured_entry_count);
 			address.setPostOfficeBox(temp);
-			if(temp.length())
+			if (temp.length())
 				fill_entry = 0;
-			temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,j,structured_entry_count,"city"));
+
+                        temp = extractStructuredField(structured_entry, "city", j, structured_entry_count);
 			address.setLocality(temp);
-			if(temp.length())
+			if (temp.length())
 				fill_entry = 0;
-			temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,j,structured_entry_count,"region"));
+
+                        temp = extractStructuredField(structured_entry, "region", j, structured_entry_count);
 			address.setRegion(temp);
-			if(temp.length())
+			if (temp.length())
 				fill_entry = 0;
-			temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,j,structured_entry_count,"postcode"));
+
+                        temp = extractStructuredField(structured_entry, "postcode", j, structured_entry_count);
 			address.setPostalCode(temp);
-			if(temp.length())
+			if (temp.length())
 				fill_entry = 0;
-			temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,j,structured_entry_count,"country"));
+
+                        temp = extractStructuredField(structured_entry, "country", j, structured_entry_count);
 			address.setCountry(temp);
-			if(temp.length())
+			if (temp.length())
 				fill_entry = 0;
-			if(fill_entry)
-			{
-				temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,j,structured_entry_count,"formattedAddress"));
+
+			if (fill_entry) {
+				temp = extractStructuredField(structured_entry, "formattedAddress", j, structured_entry_count);
 				address.setStreet(temp);
 			}
 			addressee.insertAddress(address);
 		}
-		
+
 		/* telephone */
 		for (j = 0; j < gcal_contact_get_phone_numbers_count(contact); j++) {
 			KABC::PhoneNumber number;
@@ -337,12 +373,26 @@ void GoogleContactsResource::retrieveItems( const Akonadi::Collection &collectio
 			number.setType(googlePhoneLabelToAkonadiType(gcal_contact_get_phone_number_type(contact, j)));
 			addressee.insertPhoneNumber(number);
 		}
+		/* im */
+		for (j = 0; j < gcal_contact_get_im_count(contact); j++) {
+			temp = QString::fromUtf8(gcal_contact_get_im_protocol(contact, j));
+			if (listImTypes.contains(temp)) {
+				if ( temp == QString::fromUtf8("JABBER") )
+					temp = QString::fromUtf8("xmpp");
+				addressee.insertCustom(QString::fromUtf8("messaging/").append(temp.toLower()),
+						QString::fromUtf8("All"),
+						QString::fromUtf8(gcal_contact_get_im_address(contact, j)));
+			}
+		}
 		/* profission */
 		temp = QString::fromUtf8(gcal_contact_get_profission(contact));
 		addressee.setTitle(temp);
 		/* company */
 		temp = QString::fromUtf8(gcal_contact_get_organization(contact));
 		addressee.setOrganization(temp);
+		/* occupation/profession */
+		temp = QString::fromUtf8(gcal_contact_get_occupation(contact));
+		addressee.insertCustom("KADDRESSBOOK","X-Profession",temp);
 		/* Google group membership */
 		addressee.insertCustom(QString::fromUtf8("Google"),
 					QString::fromUtf8("groupMembership_nr"),
@@ -363,7 +413,7 @@ void GoogleContactsResource::retrieveItems( const Akonadi::Collection &collectio
 		addressee.insertCustom("KADDRESSBOOK","BlogFeed",tempUrl.prettyUrl());
 		/* birthday */
 		tempDate = QDateTime::fromString(gcal_contact_get_birthday(contact),"yyyy-MM-dd");
-		if(!tempDate.isValid())
+		if (!tempDate.isValid())
 			tempDate = QDateTime::fromString(gcal_contact_get_birthday(contact),"--MM-dd");
 		addressee.setBirthday(tempDate);
 		/* photo */
@@ -400,6 +450,7 @@ void GoogleContactsResource::aboutToQuit()
 {
 	// TODO: any cleanup you need to do while there is still an active
 	// event loop. The resource will terminate after this method returns
+    gcal_final_cleanup();
 }
 
 void GoogleContactsResource::doSetOnline(bool online)
@@ -410,13 +461,19 @@ void GoogleContactsResource::doSetOnline(bool online)
 	QString password;
 	int result = 0;
 	WId window = winIdForDialogs();
+        retrieveUsername(user);
 
-	if (online)
-		if (!retrieveFromWallet(user, password, window))
-			if (!(result = authenticate(user, password))) {
-				ResourceBase::doSetOnline(online);
-				synchronize();
-			}
+	if (online) {
+            if (!user.isEmpty()) {
+
+		if (!retrieveFromWallet(user, password, window,
+                                        QString("gcont")))
+                    if (!(result = authenticate(user, password))) {
+                        ResourceBase::doSetOnline(online);
+                        synchronize();
+                    }
+            }
+        }
 
 	if (result) {
 		kError() << "Failed setting online.";
@@ -434,9 +491,12 @@ int GoogleContactsResource::getUpdated(char *timestamp)
 	gcal_contact_t contact;
 	QString newerTimestamp;
 	QString temp;
+	QStringList listImTypes;
+	listImTypes << "AIM" << "MSN" << "YAHOO" << "SKYPE" << "ICQ" << "JABBER" << "QQ";
 	QDateTime tempDate;
-	KUrl tempUrl; 
+	KUrl tempUrl;
 	KABC::Picture photo;
+	KABC::Address::Type temp_address_type;
 	QImage image;
 	gcal_structured_subvalues_t structured_entry;
 	int structured_entry_count;
@@ -481,35 +541,41 @@ int GoogleContactsResource::getUpdated(char *timestamp)
 			/* structured name */
 			structured_entry = gcal_contact_get_structured_name(contact);
 			fill_entry = 1;
-			temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,0,1,"givenName"));
+                        temp = extractStructuredField(structured_entry, "givenName");
 			addressee.setGivenName(temp);
-			if(temp.length())
+			if (temp.length())
 				fill_entry = 0;
-			temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,0,1,"additionalName"));
+
+                        temp = extractStructuredField(structured_entry, "additionalName");
 			addressee.setAdditionalName(temp);
-			if(temp.length())
+			if (temp.length())
 				fill_entry = 0;
-			temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,0,1,"familyName"));
+
+                        temp = extractStructuredField(structured_entry, "familyName");
 			addressee.setFamilyName(temp);
-			if(temp.length())
+			if (temp.length())
 				fill_entry = 0;
-			temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,0,1,"namePrefix"));
+
+
+                        temp = extractStructuredField(structured_entry, "namePrefix");
 			addressee.setPrefix(temp);
-			if(temp.length())
+			if (temp.length())
 				fill_entry = 0;
-			temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,0,1,"nameSuffix"));
+
+
+                        temp = extractStructuredField(structured_entry, "nameSuffix");
 			addressee.setSuffix(temp);
-			if(temp.length())
+			if (temp.length())
 				fill_entry = 0;
-			if(fill_entry)
-			{
-				temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,0,1,"fullName"));
+			if (fill_entry) {
+
+				temp = extractStructuredField(structured_entry, "fullName");
 				addressee.setNameFromString(temp);
-				if(temp.length())
+				if (temp.length())
 					fill_entry = 0;
 			}
-			if(fill_entry)
-			{
+
+			if (fill_entry) {
 				/* name */
 				temp = QString::fromUtf8(gcal_contact_get_title(contact));
 				addressee.setNameFromString(temp);
@@ -526,46 +592,54 @@ int GoogleContactsResource::getUpdated(char *timestamp)
 				addressee.insertCustom(QString::fromUtf8("Google"),
 						QString::fromUtf8("typeof_email_").append(QString::number(j)), temp);
 			}
-			
+
 			/* address */
 			structured_entry = gcal_contact_get_structured_address(contact);
 			structured_entry_count = gcal_contact_get_structured_address_count(contact);
 			for (j = 0; j < structured_entry_count; j++) {
 				KABC::Address address;
 				fill_entry = 1;
-				address.setType(googleAddressLabelToAkonadiType(gcal_contact_get_structured_address_type(contact,j,structured_entry_count)));
-				temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,j,structured_entry_count,"street"));
+				temp_address_type = googleAddressLabelToAkonadiType(gcal_contact_get_structured_address_type(contact, j, structured_entry_count));
+				if (j == gcal_contact_get_pref_structured_address(contact))
+					temp_address_type = temp_address_type | KABC::Address::Pref;
+				address.setType(temp_address_type);
+
+                                temp = extractStructuredField(structured_entry, "street", j, structured_entry_count);
 				address.setStreet(temp);
-				if(temp.length())
+				if (temp.length())
 					fill_entry = 0;
-				temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,j,structured_entry_count,"pobox"));
+
+                                temp = extractStructuredField(structured_entry, "pobox", j, structured_entry_count);
 				address.setPostOfficeBox(temp);
-				if(temp.length())
+				if (temp.length())
 					fill_entry = 0;
-				temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,j,structured_entry_count,"city"));
+
+                                temp = extractStructuredField(structured_entry, "city", j, structured_entry_count);
 				address.setLocality(temp);
-				if(temp.length())
+				if (temp.length())
 					fill_entry = 0;
-				temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,j,structured_entry_count,"region"));
+
+                                temp = extractStructuredField(structured_entry, "region", j, structured_entry_count);
 				address.setRegion(temp);
-				if(temp.length())
+				if (temp.length())
 					fill_entry = 0;
-				temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,j,structured_entry_count,"postcode"));
+
+                                temp = extractStructuredField(structured_entry, "postcode", j, structured_entry_count);
 				address.setPostalCode(temp);
-				if(temp.length())
+				if (temp.length())
 					fill_entry = 0;
-				temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,j,structured_entry_count,"country"));
+
+                                temp = extractStructuredField(structured_entry, "country", j, structured_entry_count);
 				address.setCountry(temp);
-				if(temp.length())
+				if (temp.length())
 					fill_entry = 0;
-				if(fill_entry)
-				{
-					temp = QString::fromUtf8(gcal_contact_get_structured_entry(structured_entry,j,structured_entry_count,"formattedAddress"));
+				if (fill_entry) {
+                                        temp = extractStructuredField(structured_entry, "formattedAddress", j, structured_entry_count);
 					address.setStreet(temp);
 				}
 				addressee.insertAddress(address);
 			}
-			
+
 			/* telephone */
 			for (j = 0; j < gcal_contact_get_phone_numbers_count(contact); j++) {
 				KABC::PhoneNumber number;
@@ -574,12 +648,28 @@ int GoogleContactsResource::getUpdated(char *timestamp)
 				number.setType(googlePhoneLabelToAkonadiType(gcal_contact_get_phone_number_type(contact, j)));
 				addressee.insertPhoneNumber(number);
 			}
+			/* im */
+			for (j = 0; j < gcal_contact_get_im_count(contact); j++) {
+				temp = QString::fromUtf8(gcal_contact_get_im_protocol(contact, j));
+				if (listImTypes.contains(temp)) {
+					if ( temp == QString::fromUtf8("JABBER") )
+						temp = QString::fromUtf8("xmpp");
+
+					addressee.insertCustom(QString::fromUtf8("messaging/").append(temp.toLower()),
+							QString::fromUtf8("All"),
+							QString::fromUtf8(gcal_contact_get_im_address(contact, j)));
+				}
+			}
+
 			/* profission */
 			temp = QString::fromUtf8(gcal_contact_get_profission(contact));
 			addressee.setTitle(temp);
 			/* company */
 			temp = QString::fromUtf8(gcal_contact_get_organization(contact));
 			addressee.setOrganization(temp);
+			/* occupation/profession */
+			temp = QString::fromUtf8(gcal_contact_get_occupation(contact));
+			addressee.insertCustom("KADDRESSBOOK","X-Profession",temp);
 			/* Google group membership */
 			addressee.insertCustom(QString::fromUtf8("Google"),
 						QString::fromUtf8("groupMembership_nr"),
@@ -589,6 +679,7 @@ int GoogleContactsResource::getUpdated(char *timestamp)
 				temp = QString::fromUtf8(gcal_contact_get_groupMembership(contact, j));
 				addressee.insertCustom(QString::fromUtf8("Google"), QString::fromUtf8("groupMembership_").append(QString::number(j)), temp);
 			}
+
 			/* note */
 			temp = QString::fromUtf8(gcal_contact_get_content(contact));
 			addressee.setNote(temp);
@@ -600,7 +691,7 @@ int GoogleContactsResource::getUpdated(char *timestamp)
 			addressee.insertCustom("KADDRESSBOOK","BlogFeed",tempUrl.prettyUrl());
 			/* birthday */
 			tempDate = QDateTime::fromString(gcal_contact_get_birthday(contact),"yyyy-MM-dd");
-			if(!tempDate.isValid())
+			if (!tempDate.isValid())
 				tempDate = QDateTime::fromString(gcal_contact_get_birthday(contact),"--MM-dd");
 			addressee.setBirthday(tempDate);
 			/* photo */
@@ -674,7 +765,10 @@ void GoogleContactsResource::configure( WId windowId )
 
 	int walletRes = saveToWallet(dlgConf->eAccount->text(),
 				     dlgConf->ePass->text(),
-				     windowId);
+				     windowId, QString("gcont"));
+        QString user(dlgConf->eAccount->text());
+        saveUsername(user);
+
 	if (walletRes)
 		kError() << "Cannot save user info: is user using kwallet?.";
 
@@ -722,18 +816,29 @@ gcal_address_type akonadiAddressTypeToGoogleLabel(KABC::Address::Type type) {
 		case KABC::Address::Work:
 			return A_WORK;
 			break;
-		case KABC::Address::Dom + 
-		  KABC::Address::Intl + 
-		  KABC::Address::Postal + 
-		  KABC::Address::Parcel +
-		  KABC::Address::Pref:
-			return A_OTHER;
-			break;
 		default:
 			return A_OTHER;
 	}
 }
 
+gcal_address_type prefAkonadiAddressTypeToGoogleLabel(KABC::Address::Type type) {
+	switch ( type ) {
+		case KABC::Address::Home + KABC::Address::Pref:
+			return A_HOME;
+			break;
+		case KABC::Address::Work + KABC::Address::Pref:
+			return A_WORK;
+			break;
+		case KABC::Address::Dom + KABC::Address::Pref:
+		case KABC::Address::Intl + KABC::Address::Pref:
+		case KABC::Address::Postal + KABC::Address::Pref:
+		case KABC::Address::Parcel + KABC::Address::Pref:
+			return A_OTHER;
+			break;
+		default:
+			return A_INVALID;
+	}
+}
 
 void GoogleContactsResource::itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection )
 {
@@ -745,12 +850,15 @@ void GoogleContactsResource::itemAdded( const Akonadi::Item &item, const Akonadi
 	gcal_contact_t contact;
 	gcal_structured_subvalues_t structured_entry;
 	int structured_entry_nr, structured_entry_count;
-	QString temp;
+	gcal_address_type temp_address_type;
+	QString temp,temp2;
 	QDateTime tempDate;
 	KUrl tempUrl;
 	QStringList listEmail;
+	QStringList listImTypes;
+	listImTypes << "aim" << "msn" << "yahoo" << "skype" << "icq" << "xmpp" << "qq";
 	QStringList::const_iterator email;
-	QByteArray t_byte;
+	QByteArray t_byte,t_byte2;
 	QList<KABC::Address> listAddress;
 	QList<KABC::PhoneNumber> listNumber;
 	KABC::Picture photo;
@@ -786,59 +894,53 @@ void GoogleContactsResource::itemAdded( const Akonadi::Item &item, const Akonadi
 	temp = addressee.givenName();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
-		gcal_contact_set_structured_entry(structured_entry,0,1, "givenName", t_byte.data());
+		gcal_contact_set_structured_entry(structured_entry, 0, 1, "givenName", t_byte.data());
 		fill_entry = 0;
 	}
 	/* Additional Name */
 	temp = addressee.additionalName();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
-		gcal_contact_set_structured_entry(structured_entry,0,1, "additionalName", t_byte.data());
+		gcal_contact_set_structured_entry(structured_entry, 0, 1, "additionalName", t_byte.data());
 		fill_entry = 0;
 	}
 	/* Family Name */
 	temp = addressee.familyName();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
-		gcal_contact_set_structured_entry(structured_entry,0,1, "familyName", t_byte.data());
+		gcal_contact_set_structured_entry(structured_entry, 0, 1, "familyName", t_byte.data());
 		fill_entry = 0;
 	}
 	/* Prefix */
 	temp = addressee.prefix();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
-		gcal_contact_set_structured_entry(structured_entry,0,1, "namePrefix", t_byte.data());
+		gcal_contact_set_structured_entry(structured_entry, 0, 1, "namePrefix", t_byte.data());
 		fill_entry = 0;
 	}
 	/* Suffix */
 	temp = addressee.suffix();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
-		gcal_contact_set_structured_entry(structured_entry,0,1, "nameSuffix", t_byte.data());
+		gcal_contact_set_structured_entry(structured_entry, 0, 1, "nameSuffix", t_byte.data());
 		fill_entry = 0;
 	}
-	
-	if(fill_entry)
+
+	if (fill_entry)
 	{
 		/* This 2 fields are required! */
 		temp = addressee.realName();
 		t_byte = temp.toUtf8();
 		gcal_contact_set_title(contact, t_byte.data());
 	}
-// 	/* All the Name Parts */
-// 	temp = addressee.realName();
-// 	if (temp.length()) {
-// 		t_byte = temp.toUtf8();
-// 		gcal_contact_set_structured_entry(structured_entry,0,1, "fullName", t_byte.data());
-// 	}
-	
+
 	/* nickname */
 	temp = addressee.nickName();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
 		gcal_contact_set_nickname(contact, t_byte.data());
 	}
-	
+
 	listEmail = addressee.emails();
 	if (!listEmail.empty()) {
 		gcal_contact_delete_email_addresses(contact);
@@ -862,7 +964,11 @@ void GoogleContactsResource::itemAdded( const Akonadi::Item &item, const Akonadi
 		structured_entry = gcal_contact_get_structured_address(contact);
 		gcal_contact_delete_structured_entry(structured_entry,gcal_contact_get_structured_address_count_obj(contact),gcal_contact_get_structured_address_type_obj(contact));
 		foreach (const KABC::Address &address, listAddress) {
-			structured_entry_nr = gcal_contact_set_structured_address_nr(contact,akonadiAddressTypeToGoogleLabel(address.type()));
+			if ((temp_address_type=prefAkonadiAddressTypeToGoogleLabel(address.type())) != A_INVALID)
+				gcal_contact_set_pref_structured_address(contact,gcal_contact_get_structured_address_count(contact));
+			else
+				temp_address_type=akonadiAddressTypeToGoogleLabel(address.type());
+			structured_entry_nr = gcal_contact_set_structured_address_nr(contact,temp_address_type);
 			structured_entry_count = gcal_contact_get_structured_address_count(contact);
 			fill_entry = 1;
 			/* Street */
@@ -872,7 +978,7 @@ void GoogleContactsResource::itemAdded( const Akonadi::Item &item, const Akonadi
 				gcal_contact_set_structured_entry(structured_entry, structured_entry_nr, structured_entry_count, "street", t_byte.data());
 				fill_entry = 0;
 			}
-			
+
 			/* PO Box */
 			temp = address.postOfficeBox();
 			if (temp.length()) {
@@ -880,7 +986,7 @@ void GoogleContactsResource::itemAdded( const Akonadi::Item &item, const Akonadi
 				gcal_contact_set_structured_entry(structured_entry, structured_entry_nr, structured_entry_count, "pobox", t_byte.data());
 				fill_entry = 0;
 			}
-			
+
 			/* City */
 			temp = address.locality();
 			if (temp.length()) {
@@ -888,7 +994,7 @@ void GoogleContactsResource::itemAdded( const Akonadi::Item &item, const Akonadi
 				gcal_contact_set_structured_entry(structured_entry, structured_entry_nr, structured_entry_count, "city", t_byte.data());
 				fill_entry = 0;
 			}
-			
+
 			/* Region */
 			temp = address.region();
 			if (temp.length()) {
@@ -896,7 +1002,7 @@ void GoogleContactsResource::itemAdded( const Akonadi::Item &item, const Akonadi
 				gcal_contact_set_structured_entry(structured_entry, structured_entry_nr, structured_entry_count, "region", t_byte.data());
 				fill_entry = 0;
 			}
-			
+
 			/* Postcode */
 			temp = address.postalCode();
 			if (temp.length()) {
@@ -904,7 +1010,7 @@ void GoogleContactsResource::itemAdded( const Akonadi::Item &item, const Akonadi
 				gcal_contact_set_structured_entry(structured_entry, structured_entry_nr, structured_entry_count, "postcode", t_byte.data());
 				fill_entry = 0;
 			}
-			
+
 			/* Country */
 			temp = address.country();
 			if (temp.length()) {
@@ -917,7 +1023,7 @@ void GoogleContactsResource::itemAdded( const Akonadi::Item &item, const Akonadi
 			if (temp.length()) {
 				t_byte = temp.toUtf8();
 				gcal_contact_set_structured_entry(structured_entry, structured_entry_nr, structured_entry_count, "formattedAddress", t_byte.data());
-				if(fill_entry)
+				if (fill_entry)
 					gcal_contact_set_structured_entry(structured_entry, structured_entry_nr, structured_entry_count, "street", t_byte.data());
 			}
 		}
@@ -936,6 +1042,26 @@ void GoogleContactsResource::itemAdded( const Akonadi::Item &item, const Akonadi
 		}
 	}
 
+	ok = 0;
+	foreach (temp, listImTypes) {
+		temp2 = addressee.custom(QString::fromUtf8("messaging/").append(temp),"All");
+		if (temp2.length())
+			ok = 1;
+	}
+	if (ok) {
+		gcal_contact_delete_im(contact);
+		foreach (temp, listImTypes) {
+			temp2 = addressee.custom(QString::fromUtf8("messaging/").append(temp),"All");
+			if (temp2.length()) {
+				if ( temp == QString::fromUtf8("xmpp") )
+					temp = QString::fromUtf8("jabber");
+				t_byte = temp.toUpper().toUtf8();
+				t_byte2 = temp2.toUtf8();
+				gcal_contact_add_im(contact, t_byte.data(), t_byte2.data(), I_OTHER, 0);
+			}
+		}
+	}
+
 	temp = addressee.title();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
@@ -948,6 +1074,12 @@ void GoogleContactsResource::itemAdded( const Akonadi::Item &item, const Akonadi
 		gcal_contact_set_organization(contact, t_byte.data());
 	}
 
+	temp = addressee.custom("KADDRESSBOOK","X-Profession");
+	if (temp.length()) {
+		t_byte = temp.toUtf8();
+		gcal_contact_set_occupation(contact, t_byte.data());
+	}
+
 	temp = addressee.custom(QString::fromUtf8("Google"), QString::fromUtf8("groupMembership_nr"));
 	num_elem = temp.toInt(&ok);
 	if (ok) {
@@ -973,14 +1105,14 @@ void GoogleContactsResource::itemAdded( const Akonadi::Item &item, const Akonadi
 		t_byte = temp.toUtf8();
 		gcal_contact_set_homepage(contact, t_byte.data());
 	}
-	
+
 	tempUrl = addressee.custom("KADDRESSBOOK","BlogFeed");
 	temp = tempUrl.prettyUrl();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
 		gcal_contact_set_blog(contact, t_byte.data());
 	}
-	
+
 	tempDate = addressee.birthday();
 	temp = tempDate.toString("yyyy-MM-dd");
 	if (temp.length()) {
@@ -1035,14 +1167,17 @@ void GoogleContactsResource::itemChanged( const Akonadi::Item &item, const QSet<
 	KABC::Addressee addressee;
 	KABC::Address address;
 	QList<KABC::Address> listAddress;
+	QStringList listImTypes;
+	listImTypes << "aim" << "msn" << "yahoo" << "skype" << "icq" << "xmpp" << "qq";
 	QStringList listEmail;
 	QStringList::const_iterator email;
 	QList<KABC::PhoneNumber> listNumber;
 	gcal_contact_t contact;
 	gcal_structured_subvalues_t structured_entry;
 	int structured_entry_nr, structured_entry_count;
-	QByteArray t_byte;
-	QString temp;
+	gcal_address_type temp_address_type;
+	QByteArray t_byte,t_byte2;
+	QString temp,temp2;
 	QDateTime tempDate;
 	KUrl tempUrl;
 	KABC::Picture photo;
@@ -1085,39 +1220,39 @@ void GoogleContactsResource::itemChanged( const Akonadi::Item &item, const QSet<
 	temp = addressee.givenName();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
-		gcal_contact_set_structured_entry(structured_entry,0,1, "givenName", t_byte.data());
+		gcal_contact_set_structured_entry(structured_entry, 0, 1, "givenName", t_byte.data());
 		fill_entry = 0;
 	}
 	/* Additional Name */
 	temp = addressee.additionalName();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
-		gcal_contact_set_structured_entry(structured_entry,0,1, "additionalName", t_byte.data());
+		gcal_contact_set_structured_entry(structured_entry, 0, 1, "additionalName", t_byte.data());
 		fill_entry = 0;
 	}
 	/* Family Name */
 	temp = addressee.familyName();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
-		gcal_contact_set_structured_entry(structured_entry,0,1, "familyName", t_byte.data());
+		gcal_contact_set_structured_entry(structured_entry, 0, 1, "familyName", t_byte.data());
 		fill_entry = 0;
 	}
 	/* Prefix */
 	temp = addressee.prefix();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
-		gcal_contact_set_structured_entry(structured_entry,0,1, "namePrefix", t_byte.data());
+		gcal_contact_set_structured_entry(structured_entry, 0, 1, "namePrefix", t_byte.data());
 		fill_entry = 0;
 	}
 	/* Suffix */
 	temp = addressee.suffix();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
-		gcal_contact_set_structured_entry(structured_entry,0,1, "nameSuffix", t_byte.data());
+		gcal_contact_set_structured_entry(structured_entry, 0, 1, "nameSuffix", t_byte.data());
 		fill_entry = 0;
 	}
-	
-	if(fill_entry)
+
+	if (fill_entry)
 	{
 		/* This 2 fields are required! */
 		temp = addressee.realName();
@@ -1130,14 +1265,14 @@ void GoogleContactsResource::itemChanged( const Akonadi::Item &item, const QSet<
 // 		t_byte = temp.toUtf8();
 // 		gcal_contact_set_structured_entry(structured_entry,0,1, "fullName", t_byte.data());
 // 	}
-	
+
 	/* nickname */
 	temp = addressee.nickName();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
 		gcal_contact_set_nickname(contact, t_byte.data());
 	}
-	
+
 	listEmail = addressee.emails();
 	if (!listEmail.empty()) {
 		gcal_contact_delete_email_addresses(contact);
@@ -1161,7 +1296,11 @@ void GoogleContactsResource::itemChanged( const Akonadi::Item &item, const QSet<
 		structured_entry = gcal_contact_get_structured_address(contact);
 		gcal_contact_delete_structured_entry(structured_entry,gcal_contact_get_structured_address_count_obj(contact),gcal_contact_get_structured_address_type_obj(contact));
 		foreach (const KABC::Address &address, listAddress) {
-			structured_entry_nr = gcal_contact_set_structured_address_nr(contact,akonadiAddressTypeToGoogleLabel(address.type()));
+			if ((temp_address_type=prefAkonadiAddressTypeToGoogleLabel(address.type())) != A_INVALID)
+				gcal_contact_set_pref_structured_address(contact,gcal_contact_get_structured_address_count(contact));
+			else
+				temp_address_type=akonadiAddressTypeToGoogleLabel(address.type());
+			structured_entry_nr = gcal_contact_set_structured_address_nr(contact,temp_address_type);
 			structured_entry_count = gcal_contact_get_structured_address_count(contact);
 			fill_entry = 1;
 			/* Street */
@@ -1171,7 +1310,7 @@ void GoogleContactsResource::itemChanged( const Akonadi::Item &item, const QSet<
 				gcal_contact_set_structured_entry(structured_entry, structured_entry_nr, structured_entry_count, "street", t_byte.data());
 				fill_entry = 0;
 			}
-			
+
 			/* PO Box */
 			temp = address.postOfficeBox();
 			if (temp.length()) {
@@ -1179,7 +1318,7 @@ void GoogleContactsResource::itemChanged( const Akonadi::Item &item, const QSet<
 				gcal_contact_set_structured_entry(structured_entry, structured_entry_nr, structured_entry_count, "pobox", t_byte.data());
 				fill_entry = 0;
 			}
-			
+
 			/* City */
 			temp = address.locality();
 			if (temp.length()) {
@@ -1187,7 +1326,7 @@ void GoogleContactsResource::itemChanged( const Akonadi::Item &item, const QSet<
 				gcal_contact_set_structured_entry(structured_entry, structured_entry_nr, structured_entry_count, "city", t_byte.data());
 				fill_entry = 0;
 			}
-			
+
 			/* Region */
 			temp = address.region();
 			if (temp.length()) {
@@ -1195,7 +1334,7 @@ void GoogleContactsResource::itemChanged( const Akonadi::Item &item, const QSet<
 				gcal_contact_set_structured_entry(structured_entry, structured_entry_nr, structured_entry_count, "region", t_byte.data());
 				fill_entry = 0;
 			}
-			
+
 			/* Postcode */
 			temp = address.postalCode();
 			if (temp.length()) {
@@ -1203,7 +1342,7 @@ void GoogleContactsResource::itemChanged( const Akonadi::Item &item, const QSet<
 				gcal_contact_set_structured_entry(structured_entry, structured_entry_nr, structured_entry_count, "postcode", t_byte.data());
 				fill_entry = 0;
 			}
-			
+
 			/* Country */
 			temp = address.country();
 			if (temp.length()) {
@@ -1216,7 +1355,7 @@ void GoogleContactsResource::itemChanged( const Akonadi::Item &item, const QSet<
 			if (temp.length()) {
 				t_byte = temp.toUtf8();
 				gcal_contact_set_structured_entry(structured_entry, structured_entry_nr, structured_entry_count, "formattedAddress", t_byte.data());
-				if(fill_entry)
+				if (fill_entry)
 					gcal_contact_set_structured_entry(structured_entry, structured_entry_nr, structured_entry_count, "street", t_byte.data());
 			}
 		}
@@ -1235,6 +1374,27 @@ void GoogleContactsResource::itemChanged( const Akonadi::Item &item, const QSet<
 		}
 	}
 
+	ok = 0;
+	foreach (temp, listImTypes) {
+		temp2 = addressee.custom(QString::fromUtf8("messaging/").append(temp),"All");
+		if (temp2.length())
+			ok = 1;
+	}
+
+	if (ok) {
+		gcal_contact_delete_im(contact);
+		foreach (temp, listImTypes) {
+			temp2 = addressee.custom(QString::fromUtf8("messaging/").append(temp),"All");
+			if (temp2.length()) {
+				if ( temp == QString::fromUtf8("xmpp") )
+					temp = QString::fromUtf8("jabber");
+				t_byte = temp.toUpper().toUtf8();
+				t_byte2 = temp2.toUtf8();
+				gcal_contact_add_im(contact, t_byte.data(), t_byte2.data(),I_OTHER, 0);
+			}
+		}
+	}
+
 	temp = addressee.title();
 	if (temp.length()) {
 		t_byte = temp.toUtf8();
@@ -1247,6 +1407,12 @@ void GoogleContactsResource::itemChanged( const Akonadi::Item &item, const QSet<
 		gcal_contact_set_organization(contact, t_byte.data());
 	}
 
+	temp = addressee.custom("KADDRESSBOOK","X-Profession");
+	if (temp.length()) {
+		t_byte = temp.toUtf8();
+		gcal_contact_set_occupation(contact, t_byte.data());
+	}
+
 	temp = addressee.custom(QString::fromUtf8("Google"), QString::fromUtf8("groupMembership_nr"));
 	num_elem = temp.toInt(&ok);
 	if (ok) {
@@ -1272,7 +1438,7 @@ void GoogleContactsResource::itemChanged( const Akonadi::Item &item, const QSet<
 		t_byte = temp.toUtf8();
 		gcal_contact_set_homepage(contact, t_byte.data());
 	}
-	
+
 	tempUrl = addressee.custom("KADDRESSBOOK","BlogFeed");
 	temp = tempUrl.prettyUrl();
 	if (temp.length()) {
